""" NewBuiltins - Auto-install the collection of new builtins provided
                  by mx.Tools

    Redirection module; use mx.Tools.NewBuiltins instead !!!

    Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.
"""
import mx.Tools.NewBuiltins
